/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var app = {
	// Settings
	//
	username:				"",
//	appService:				"http://admintest.tyresandservice.co.uk/AppServices.asmx/",
//	appService:				"http://192.168.0.108:53995/AppServices.asmx/",
	appService:				"https://admin.tyresandservice.co.uk/AppServices.asmx/",
	imageStore:				"http://images.tyresandservice.co.uk/",
	ajaxTimeout:			10000, // ten seconds
	
	// Account
	//
	tradingName:			"",
	account:				"",
	website:				"",
	
	// Application data
	//
	appToken:				"",
	orders:					"",
	currentOrderID:			-1,
	currentOrderItems:		"",
	currentOrderItemIndex:	-1,
	

    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
	//
    bindEvents: function() {
        document.addEventListener('deviceready', this.deviceReady, false);
    },

	clearData: function() {
		localStorage.clear();
	},
	
	// deviceReady - handle the device read event and set up the menu page
	//
	deviceReady: function() {
		// Set up pageshow handlers
		//
		$("#pgHome").bind("pageshow", function() { app.showOrders(); });
		$("#pgOrderDetails").bind("pageshow", function() { app.showOrderDetails(); });
		$("#pgTyreDetails").bind("pageshow", function() { app.showTyreDetails(); });
		$("#pgSettings").bind("pageshow", function() { app.showAccountSettings(); });

		// Add a gap above the status bar for iOS devices
		//
		if (device.platform.toLowerCase() == "ios")
		{
			$(".tsHeader").css("padding-top", "25px");
			$(".tsHeader a").css("margin-top", "25px");
		}

		if (app.username == "" || app.appToken == "")
		{
			app.showMenuOptions(false);
			return;
		}

		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
        $.ajax({
            type:			"POST",
            data:			"{'appToken':'" + app.appToken + "','username':'" + app.username + "'}",
            url:			app.appService + "CheckToken",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
			timeout:		app.ajaxTimeout,
            dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								if (data.ok == "true")
								{
									// Connected and token valid
									//
									app.showMenuOptions(true);
									navigator.splashscreen.hide();
									$.mobile.loading('hide');	
								}
								else
								{
									// Connected but token invalid/expired
									//
									app.appToken = "";
									localStorage.appToken = "";

									app.showMenuOptions(false);
									navigator.splashscreen.hide();
									$.mobile.loading('hide');	
								}
                            },
            error:		function(request, textStatus, message){
								console.log("checkToken failed: [" + textStatus + "] " + message);

								if (request.status == 0 && (app.orders != null || app.orders != ""))
								{
									// No connection but there are orders - let the user continue as best they can
									//
									app.showMenuOptions(true);
								}
								else
									app.showMenuOptions(false);

								navigator.splashscreen.hide();
								$.mobile.loading('hide');	
                            }
        });	
	},
	
	// Download account details
	//
	downloadAccountDetails: function(orderID) {
		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
        $.ajax({
            type:			"POST",
            data:			"{'appToken':'" + app.appToken + "','username':'" + app.username + "'}",
            url:			app.appService + "AccountDetails",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
 			timeout:		app.ajaxTimeout,
			dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								app.tradingName = data.TradingName;
								app.account = data.MicheldeverAccount;
								app.website = data.DomainName;
								
								// Save the data
								//
								localStorage.tradingName = app.tradingName;
								localStorage.account = app.account;
								localStorage.website = app.website;

								app.receivedEvent("accountdetailsdownloaded");
								$.mobile.loading('hide');	
                            },
			error:			function(request, textStatus, message){
								$.mobile.loading('hide');	
								console.log("downloadAccountDetails failed: [" + textStatus + "] " + message);
								
								if (request.status == 0)
									app.receivedEvent("offline");
                             
							}							
        });		
	},

	// Download the items for an order
	//
	downloadOrderItems: function(orderID) {
		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
        $.ajax({
            type:			"POST",
            data:			"{'appToken':'" + app.appToken + "','username':'" + app.username + "','orderID':'" + orderID + "'}",
            url:			app.appService + "GetOrderItems",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
 			timeout:		app.ajaxTimeout,
			dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								app.currentOrderItems = data;
								app.receivedEvent("orderitemsdownloaded");
								$.mobile.loading('hide');	
                            },
			error:		function(request, textStatus, message){
								$.mobile.loading('hide');	
								console.log("downloadOrderItems failed: [" + textStatus + "] " + message);
								
								if (request.status == 0)
									app.receivedEvent("offline");
                             
							}							
        });		
	},

	// Download the list of dealer orders
	//
	downloadOrders: function() {
		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
        $.ajax({
            type:			"POST",
            data:			"{'appToken':'" + app.appToken + "','username':'" + app.username + "'}",
            url:			app.appService + "ListOrders",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
 			timeout:		app.ajaxTimeout,
			dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								app.orders = data;
								localStorage.orders = JSON.stringify(app.orders);
								app.receivedEvent("ordersdownloaded");
								$.mobile.loading('hide');	
                            },
			error:			function(request, textStatus, message){
								$.mobile.loading('hide');	
								console.log("downloadOrders failed: [" + textStatus + "] " + message);

								if (request.status == 0)
									app.receivedEvent("offline");
                             }							
         });		
	},
	
	// filterData - filter data returned from AJAX call
	//
	filterData: function(data) {
		var filteredData = eval("(" + data + ")");
		if (filteredData.hasOwnProperty("d"))
			return filteredData.d;
		else
			return filteredData;
	},
	
	getLocalStorageItem: function(key) {
		var res = localStorage.getItem(key);
		if (res == null)
			return "";
		else
			return res;
	},
	
	// Get an order from the stored list
	//
	getOrder: function(orderID) {
		var order = null;
		$.each(app.orders, function(i, item) {
			if (item.OrderID == orderID)
				order = item;
		});
		return order;
	},
	
	// Get an item from the current order
	//
	getOrderItem: function(index) {
		var orderItem = null;
		$.each(app.currentOrderItems, function(i, item) {
			if (item.Index == index)
				orderItem = item;
		});
		return orderItem;
	},

    // Application Constructor
    initialiseApp: function() {
        this.bindEvents();
		
		// Load account and order data
		//
		app.username = app.getLocalStorageItem("username");
		app.appToken = app.getLocalStorageItem("appToken");
		if (app.getLocalStorageItem("orders") != "")
			app.orders = app.filterData(app.getLocalStorageItem("orders"));
		app.tradingName = app.getLocalStorageItem("tradingName");
		app.account = app.getLocalStorageItem("account");
		app.website = app.getLocalStorageItem("website");
    },
	
	// listOrderItems - list all dealer's orders
	//
	listOrderItems: function() {
		$("#orderItemsList").empty();
		
		$.each(app.currentOrderItems, function(i, item) {
			var tyreName = item.ManufacturerName + ((item.Pattern != null)? " " + item.Pattern : "");
			
			$("#orderItemsList").append("<li>" +
										"<a href=\"#\" class=\"orderItem\" data-value=\"" + item.Index + "\">" +
											"<h2>" + item.Quantity + " x " + tyreName + "</h2>" +
											"<h3>" + item.Width + "/" + item.Profile + "R" + item.Rim + " " + item.Speed + item.LoadIndex + "</h3>" +
											"<p>" + item.MicheldeverStockCode + "</p>" +
											"<p class=\"ui-li-aside\"><strong>&pound;" + item.Price.toFixed(2) + "</strong></p>" +
										"</a>" +
									"</li>");
			
		});
		$("#orderItemsList").listview('refresh');

		$(".orderItem").bind('click', function() {
			app.currentOrderItemIndex = $(this).attr('data-value');
			console.log(app.currentOrderItemIndex + " clicked");
			$.mobile.changePage("#pgTyreDetails");
		});
		
	},
	
	// listOrders - list all dealer's orders
	//
	listOrders: function() {
		$("#ordersList1").empty();
		$("#ordersList2").empty();
		$("#ordersList3").empty();
		$("#ordersList4").empty();
	
		var count1 = 0;
		var count2 = 0;
		var count3 = 0;
		var count4 = 0;
		
		$.each(app.orders, function(i, item) {
			var listItem =
				"<li>" + 
					"<a href=\"#\" class=\"orderLink\" data-value=\"" + item.OrderID + "\">" +
						"<h3>" + item.OrderRef + ((item.Comment != null && item.Comment !="")? "<img src=\"img/comment-white.png\" alt=\"comment\" />" : "") + "</h3>" +
						"<p>" + item.Customer + "</p>" +
						"<p>" + item.OrderDate + ", " + app.orderStatus(item.Status) + "</p>" +
						"<p class=\"ui-li-aside\"><strong>&pound;" + item.TotalAmount.toFixed(2) + "</strong></p>" +
					"</a>" +
				"</li>";
				
			if (item.Status == 1)
			{
				$("#ordersList1").append(listItem);
				count1++;
			}
			else if (item.Status == 2)
			{
				$("#ordersList2").append(listItem);
				count2++;
			}
			else if (item.Status == 3)
			{
				$("#ordersList3").append(listItem);
				count3++;
			}
			else if (item.Status == 4)
			{
				$("#ordersList4").append(listItem);
				count4++;
			}
				
		});
			
		$("#ordersList1").listview('refresh');
		$("#ordersList2").listview('refresh');
		$("#ordersList3").listview('refresh');
		$("#ordersList4").listview('refresh');
		
		$("#ordersHead1 .ui-li-count").text(count1);
		$("#ordersHead2 .ui-li-count").text(count2);
		$("#ordersHead3 .ui-li-count").text(count3);
		$("#ordersHead4 .ui-li-count").text(count4);
		
		$(".orderLink").bind('click', function() {
			app.currentOrderID = $(this).attr('data-value');
			console.log(app.currentOrderID + " clicked");
			$.mobile.changePage("#pgOrderDetails");
		});
	},
	
	// login - to Tyres and Service
	//
    login: function() {
		var username = $("#username").val();
		var password = $("#password").val();
		
		if (username == "" || password == "")
			return;
		
		// Save username for next time around
		//
		app.username = username;
		localStorage.username = app.username;

		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
        $.ajax({
            type:			"POST",
            data:			"{'username':'" + app.username + "','password':'" + password + "'}",
            url:			app.appService + "Login",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
			timeout:		app.ajaxTimeout,
            dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								if (data.appToken == "")
								{
									// Login failed
									//
									$.mobile.loading('hide');
									navigator.notification.alert("Check your username and password and try again.", function() {}, "Login Failed", "OK");
								}
								else
								{
									// Login successful, save the app token
									//
									app.appToken = data.appToken;
									localStorage.appToken = app.appToken;
									
									// Kick off an orders download
									//
									app.receivedEvent("loggedin");
									
									// Setup the menu
									//
									app.showMenuOptions(true);
									$.mobile.changePage("#pgMenu");
									$.mobile.loading('hide');
								}
                            },
           error:		function(request, textStatus, message){
								$.mobile.loading('hide');	
								console.log("login failed: [" + textStatus + "] " + message);
								
								if (request.status == 0)
									app.receivedEvent("offline");

								app.showMenuOptions(false);
                            }							
        });	
	},

	logout:	function()
	{
		app.clearData();
		app.showMenuOptions(false);
	},
	
	offline: function() {
		navigator.notification.alert("Can't connect to TyreClick servers.  Check your network status and re-try.", function() {}, "Connection Error", "OK");
	},
	
 	// orderStatus - readable version of the order status
	//
	orderStatus: function(s) {
		switch (s)
		{
			case 1:
				return "Waiting for tyres";
				break;
			case 2:
				return "Tyres arrived / waiting for customer";
				break;
			case 3:
				return "Fitted / complete";
				break;
			case 4:
				return "Cancelled";
				break;
			default:
				console.log("Unknown status");
				return "unknown";
				break;
		}

	},
	
	// recievedEvent - handle all events
	//
    receivedEvent: function(id) {
        console.log("Received Event: " + id);
		switch (id)
		{
			case "deviceready":
				app.checkAppToken();
				break;
			case "loggedin":
				app.downloadAccountDetails();
				break;
			case "accountdetailsdownloaded":
				app.downloadOrders();
				break;
			case "offline":
				app.offline();
				break;
			case "ordersdownloaded":
				app.listOrders();
				break;
			case "orderitemsdownloaded":
				app.listOrderItems();
				break;
			default:
				console.log("Unexpected event");
				break;
		}
    },
	
	saveStatus: function() {
		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});

		var newStatus = parseInt($("input:radio[name='odOrderStatus']:checked").val());

        $.ajax({
            type:			"POST",
            data:			"{'appToken':'" + app.appToken + "','username':'" + app.username + "','orderID':'" + app.currentOrderID + "','orderStatus':'" + newStatus + "'}",
            url:			app.appService + "SetOrderStatus",
            contentType:	"application/json; charset=utf-8",
            dataType:		"json",
			timeout:		app.ajaxTimeout,          
            dataFilter:		function(data){
								return app.filterData(data);
                            },
            success:		function(data){
								// Update the local data  - rather than re-download it
								//
								$.each(app.orders, function(i, item) {
									if (item.OrderID == app.currentOrderID)
										item.Status = newStatus;
								});

								// Save for later
								//
								localStorage.orders = app.orders;
			
								// Update lists and searches
								//
								app.listOrders();
								app.search();
								
								app.receivedEvent("orderstatuschanged");
								$.mobile.loading('hide');	
                            },
           error:		function(request, textStatus, message){
								$.mobile.loading('hide');	
								console.log("saveStatus failed: [" + textStatus + "] " + message);
								
								if (request.status == 0)
									app.receivedEvent("offline");
								
                            }
        });		
	},

	// listOrders - list all dealer's orders
	//
	search: function() {
		$("#searchList").empty();
	
		var searchTerm = $("#searchTerm").val();
		if (searchTerm == "")
			return;

		$.mobile.loading('show', {theme:"a", text:"", textonly:false, textVisible: false});
		
		var results = 0;
		searchTerm = searchTerm.toLowerCase();
		$.each(app.orders, function(i, item) {
			if (item.OrderRef.toLowerCase().indexOf(searchTerm) >= 0 ||
				item.Customer.toLowerCase().indexOf(searchTerm) >= 0 ||
				item.PostCode.toLowerCase().indexOf(searchTerm) >= 0 ||
				(item.Comment != null && item.Comment.toLowerCase().indexOf(searchTerm) >= 0))
			{
				var listItem =
					"<li>" + 
						"<a href=\"#\" class=\"searchLink\" data-value=\"" + item.OrderID + "\">" +
							"<h3>" + item.OrderRef + "</h3>" +
							"<p>" + item.Customer + "</p>" +
							"<p>" + item.OrderDate + ", " + app.orderStatus(item.Status) + "</p>" +
							"<p class=\"ui-li-aside\"><strong>&pound;" + item.TotalAmount.toFixed(2) + "</strong></p>" +
						"</a>" +
					"</li>";
				
				$("#searchList").append(listItem);
				results++;
			}
		});
			
		$("#searchList").listview('refresh');
		
		$(".searchLink").bind('click', function() {
			app.currentOrderID = $(this).attr('data-value');
			console.log(app.currentOrderID + " clicked");
			$.mobile.changePage("#pgOrderDetails");
		});
		
		$.mobile.loading('hide');	

		if (results == 0)
			navigator.notification.alert("No bookings match your search", function() {}, "Search", "OK");
	},

	showAccountSettings: function() {
		$("#sTradingName").text(app.tradingName);
		$("#sAccount").text(app.account);
		$("#sUsername").text(app.username);
		$("#sWebsite").text(app.website);
	},

	showMenuOptions: function(loggedIn) {
		if (loggedIn)
		{
			$("#menuLogin").addClass("ui-disabled");
			$("#menuHome").removeClass("ui-disabled");
			$("#menuSettings").removeClass("ui-disabled");
			$("#menuWebsite").removeClass("ui-disabled");
			$("#menuLogout").removeClass("ui-disabled");
		}
		else
		{
			$("#menuLogin").removeClass("ui-disabled");
			$("#menuHome").addClass("ui-disabled");
			$("#menuSettings").addClass("ui-disabled");
			$("#menuWebsite").addClass("ui-disabled");
			$("#menuLogout").addClass("ui-disabled");
		}
	},
	
	showOrderDetails: function() {
		var order = app.getOrder(app.currentOrderID);
		if (order != null)
		{
			$("#ohName").text(order.Customer);
			$("#ohPostCode").text(order.PostCode);

			$("#odOrderRef").text(order.OrderRef);
			$("#odOrderStatus1").prop("checked", (order.Status == 1));
			$("#odOrderStatus2").prop("checked", (order.Status == 2));
			$("#odOrderStatus3").prop("checked", (order.Status == 3));
			$("#odOrderStatus4").prop("checked", (order.Status == 4));

			$("#odOrderStatus1").prop("disabled", (order.Status > 1));
			((order.Status > 1)? $('[for="odOrderStatus1"]').addClass("ui-disabled") : $('[for="odOrderStatus1"]').removeClass("ui-disabled"));
			
			$("#odOrderStatus2").prop("disabled", (order.Status > 2));
			((order.Status > 2)? $('[for="odOrderStatus2"]').addClass("ui-disabled") : $('[for="odOrderStatus2"]').removeClass("ui-disabled"));

			$("#odOrderStatus3").prop("disabled", (order.Status == 4));
			((order.Status == 4)? $('[for="odOrderStatus3"]').addClass("ui-disabled") : $('[for="odOrderStatus3"]').removeClass("ui-disabled"));

			$("#odOrderStatus4").prop("disabled", (order.Status == 3));
			((order.Status == 3)? $('[for="odOrderStatus4"]').addClass("ui-disabled") : $('[for="odOrderStatus4"]').removeClass("ui-disabled"));

			$("input[type='radio']").checkboxradio("refresh");
				
			$("#odOrderDate").text(order.OrderDate);
			$("#odFitting").text(order.FittingDate + " after " + order.FittingTime);

			$("#odTotal").text("£" + order.TotalAmount);
			$("#odComments").text(order.Comment == null? "none" : order.Comment);

			app.downloadOrderItems(app.currentOrderID);
		}
	},
	
	showOrders: function() {
		app.listOrders();
	},
	
	showTyreDetails: function() {
		var item = app.getOrderItem(app.currentOrderItemIndex);
		if (item != null)
		{
			$("#tdImage").attr("src", app.imageStore + item.ImageUrl);
			$("#tdName").text(item.ManufacturerName + " " + item.Pattern);
			var description = "";
			if (item.Strapline != "")
				description += "<p class=\"strapline\">" + item.Strapline + "</p>";
			
			if (item.Description != "")
				description += item.Description;
			
			$("#tdDescription").html(description);
		}
	},
	
	showWebsite: function(admin) {
		if (admin)
			window.open("https://admin.tyresandservice.co.uk", "_system");
		else
			window.open("http://" + app.website, "_system");
	}
};


